//--------------------------------------------------------------------------------
// Project       : SWI-Speech
// File          : testswi.cpp.pl
// Purpose       : Prolog Interface to SAPI speech recognition and synthesis engine.
// System        : SWI-Prolog 5.0.8, MS SAPI 5.0
// Author        : Jonathan T. McClain
// Institution   : The University of Georgia Artificial Intelligence Center
// Last Modified : 5/8/03
//--------------------------------------------------------------------------------

#include "stdafx.h"
#include <strstream>
#include <sapi.h>
#include <string.h>
#include <stdio.h>
#include <atlbase.h>
#include <windows.h>
#include "sphelper.h"
#include <swi-cpp.h>
#include <iostream.h>

#define PROLOG_MODULE swi2excel

inline HRESULT BlockForResult(ISpRecoContext * pRecoCtxt, ISpRecoResult ** ppResult)
{
    HRESULT hr = S_OK;
	CSpEvent event;

    while (SUCCEEDED(hr) &&
           SUCCEEDED(hr = event.GetFrom(pRecoCtxt)) &&
           hr == S_FALSE)
    {
        hr = pRecoCtxt->WaitForNotifyEvent(INFINITE);
    }

    *ppResult = event.RecoResult();
    if (*ppResult)
    {
        (*ppResult)->AddRef();
    }

    return hr;
}

// print_spec(+Text)
//  This predicate prints a line of text with the word "hello" in front.
PREDICATE(print_spec,1)
{
	char * Name = A1;
	cout <<"Hello "<<Name<<endl;
	return TRUE;
}

// swi_speak(+Text)
//  This predicate speaks a line of text using the default voice settings as set by the user.
PREDICATE(swi_speak,1)
{
    ISpVoice * pVoice = NULL;
	CSpDynamicString mystring;

    if (FAILED(::CoInitialize(NULL)))
        return FALSE;

    HRESULT hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);
    if( SUCCEEDED( hr ) )
    {
		mystring = A1;
        hr = pVoice->Speak(mystring, 0, NULL);
        pVoice->Release();
        pVoice = NULL;
    }

    ::CoUninitialize();
    return TRUE;
}

// swi_listen(-Text)
//  This predicate uses SAPI's speech recognition to listen for and return a line of text.
PREDICATE(swi_listen,1)
{
	HRESULT hr;
    if (SUCCEEDED(hr = ::CoInitialize(NULL)))
    {
        {
            CComPtr<ISpRecoContext> cpRecoCtxt;
            CComPtr<ISpRecoGrammar> cpGrammar;
            CComPtr<ISpVoice> cpVoice;
            hr = cpRecoCtxt.CoCreateInstance(CLSID_SpSharedRecoContext);
            if (cpRecoCtxt &&
                SUCCEEDED(hr = cpRecoCtxt->SetNotifyWin32Event()) &&
                SUCCEEDED(hr = cpRecoCtxt->SetInterest(SPFEI(SPEI_RECOGNITION), SPFEI(SPEI_RECOGNITION))) &&
                SUCCEEDED(hr = cpRecoCtxt->SetAudioOptions(SPAO_RETAIN_AUDIO, NULL, NULL)) &&
                SUCCEEDED(hr = cpRecoCtxt->CreateGrammar(0, &cpGrammar)) &&
                SUCCEEDED(hr = cpGrammar->LoadDictation(NULL, SPLO_STATIC)) &&
                SUCCEEDED(hr = cpGrammar->SetDictationState(SPRS_ACTIVE)))
            {
                CComPtr<ISpRecoResult> cpResult;

                while (SUCCEEDED(hr = BlockForResult(cpRecoCtxt, &cpResult)))
                {
                    cpGrammar->SetDictationState( SPRS_INACTIVE );

                    CSpDynamicString dstrText;
					char * mystring;

                    if (SUCCEEDED(cpResult->GetText(SP_GETWHOLEPHRASE, SP_GETWHOLEPHRASE, 
                                                    TRUE, &dstrText, NULL)))
                    {
						mystring = dstrText.CopyToChar();
                       cpResult.Release();
					   return (A1 = mystring);
                    }
					cpGrammar->SetDictationState( SPRS_ACTIVE );
                } 
            }
        }
        ::CoUninitialize();

	}
}

// swi_speak(+Text, +Rate, +Volume, +ReqAttributes, +OptAttributes)
//  This predicate uses SpFindBestToken to search for a voice that best matches
//  the input attributes and uses that voice to speak the line of text.
PREDICATE(swi_speak,5)
{
    ISpVoice * pVoice = NULL;
	ISpObjectToken * pCurVoiceToken;
	CSpDynamicString mystring, required, optional;
	WCHAR * wrequired, * woptional;
	int rate, volume;
	HRESULT hr = TRUE;

    if (FAILED(::CoInitialize(NULL)))
        return FALSE;

	required = A4;
	wrequired = required.Copy();

	optional = A5;
	woptional = optional.Copy();

	hr = SpFindBestToken(SPCAT_VOICES, wrequired, woptional, &pCurVoiceToken);

	hr = CoCreateInstance(CLSID_SpVoice, NULL, CLSCTX_ALL, IID_ISpVoice, (void **)&pVoice);

    if( SUCCEEDED( hr ) )
    {
		hr = pVoice->SetVoice(pCurVoiceToken);
		mystring = A1;
		rate = A2;
		volume = A3;
		if((rate < -10) || (rate > 10))
		{
			rate = 0;
		};
		if((volume < 0) || (volume > 100))
		{
			volume = 50;
		};
		hr = pVoice->SetRate(rate);
		hr = pVoice->SetVolume(volume);
        hr = pVoice->Speak(mystring, 0, NULL);
        pVoice->Release();
        pVoice = NULL;
    }

    ::CoUninitialize();
    return TRUE;          

}